"""
Module that contains the PropertiesWindow class
"""
# This file is part of 'TuiView' - a simple Raster viewer
# Copyright (C) 2012  Sam Gillingham
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
from PyQt5.QtWidgets import QDialog, QVBoxLayout, QPushButton
from PyQt5.QtWidgets import QGroupBox, QLabel, QGridLayout, QTabWidget, QWidget
from PyQt5.QtWidgets import QComboBox

from . import plotwidget


class PropertiesWindow(QDialog):
    def __init__(self, parent, info):
        QDialog.__init__(self, parent)
        self.info = info

        self.mainLayout = QVBoxLayout()

        self.fileLayout = QVBoxLayout()
        self.fileTab = QTabWidget()

        self.fileInfoWidget = QWidget()
        self.fileInfoWidgetLayout = QGridLayout()
        self.fileInfoWidget.setLayout(self.fileInfoWidgetLayout)
        for rowCount, values in enumerate(info.fileInfo):
            name, value = values
            nameLabel = QLabel()
            nameLabel.setText(name)
            self.fileInfoWidgetLayout.addWidget(nameLabel, rowCount, 0)

            valueLabel = QLabel()
            valueLabel.setText(value)
            self.fileInfoWidgetLayout.addWidget(valueLabel, rowCount, 1)

        self.fileTab.addTab(self.fileInfoWidget, "File Information")

        self.fileProjWidget = QWidget()
        self.fileProjWidgetLayout = QGridLayout()
        self.fileProjWidget.setLayout(self.fileProjWidgetLayout)

        coordInfo = [('Projection', info.getProjection()), 
            ('Datum', info.getDatum()),
            ('Spheroid', info.getSpheroid()),
            ('Units', info.getUnits()),
            ('UTM Zone', info.getUTMZone())]
        for rowCount, values in enumerate(coordInfo):
            name, value = values
            nameLabel = QLabel()
            nameLabel.setText(name)
            self.fileProjWidgetLayout.addWidget(nameLabel, rowCount, 0)

            valueLabel = QLabel()
            if value is not None:
                valueLabel.setText(value)
            else:
                valueLabel.setText("Not Set")
            self.fileProjWidgetLayout.addWidget(valueLabel, rowCount, 1)

        self.fileTab.addTab(self.fileProjWidget, "Coordinate System")

        self.fileLayout.addWidget(self.fileTab)
        
        self.fileGroup = QGroupBox('File')
        self.fileGroup.setLayout(self.fileLayout)
        self.mainLayout.addWidget(self.fileGroup)

        self.layerLayout = QVBoxLayout()
        self.layerTab = QTabWidget()

        self.layerSelLayout = QGridLayout()
        self.layerSelLabel = QLabel()
        self.layerSelLabel.setText("Layer:")
        self.layerSelLayout.addWidget(self.layerSelLabel, 0, 0)
        self.layerSelCombo = QComboBox()
        for idx, name in enumerate(info.bandNames):
            self.layerSelCombo.addItem(name, idx)
        self.layerSelCombo.setEnabled(len(info.bandNames) > 1)

        self.layerSelLayout.addWidget(self.layerSelCombo, 0, 1)
        self.layerSelCombo.currentIndexChanged.connect(self.layerChanged)

        self.layerLayout.addLayout(self.layerSelLayout)
        self.layerLayout.addWidget(self.layerTab)

        self.layerInfoWidget = QWidget()
        self.plotWidget = plotwidget.PlotBarWidget(self)

        self.layerInfoLayout = QGridLayout()
        self.layerValueLabels = {}  # so we can just update what we need
        self.layerChanged(0)  # fill it in
        self.layerSelCombo.setCurrentIndex(0)  # make sure we are at first one
        self.layerInfoWidget.setLayout(self.layerInfoLayout)

        self.layerTab.addTab(self.layerInfoWidget, 'Layer Information')

        self.layerTab.addTab(self.plotWidget, 'Layer Histogram')

        self.layerGroup = QGroupBox('Layer')
        self.layerGroup.setLayout(self.layerLayout)

        self.mainLayout.addWidget(self.layerGroup)

        self.bandInfoLayout = QGridLayout()

        self.mainLayout.addLayout(self.bandInfoLayout)

        self.closeButton = QPushButton()
        self.closeButton.setText("Close")
        self.closeButton.clicked.connect(self.accept)

        self.mainLayout.addWidget(self.closeButton)

        self.setLayout(self.mainLayout)
        self.setSizeGripEnabled(True)
        self.resize(500, 600)

    def layerChanged(self, comboIdx):
        """
        Called when the band changed - update GUI
        """
        # get the idx in the info object
        idx = self.layerSelCombo.itemData(comboIdx)

        if len(self.layerValueLabels) == 0:
            for rowCount, values in enumerate(self.info.bandInfo[idx]):
                name, value = values
                nameLabel = QLabel()
                nameLabel.setText(name)
                self.layerInfoLayout.addWidget(nameLabel, rowCount, 0)

                valueLabel = QLabel()
                valueLabel.setText(value)
                self.layerInfoLayout.addWidget(valueLabel, rowCount, 1)
                self.layerValueLabels[name] = valueLabel

        else:
            for name, value in self.info.bandInfo[idx]:
                valueLabel = self.layerValueLabels[name]
                valueLabel.setText(value)

        # update histogram info
        name = self.info.bandNames[idx]
        if name in self.info.histograms:
            minVal, maxVal, histData = self.info.histograms[name]

            plotBar = plotwidget.PlotBars(histData, minVal, maxVal)
            self.plotWidget.setBars(plotBar)
        else:
            # no histo info
            self.plotWidget.setBars(None)





"""
Module that contains the PlotScalingDialog class
"""
# This file is part of 'TuiView' - a simple Raster viewer
# Copyright (C) 2012  Sam Gillingham
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from PyQt5.QtWidgets import QDialog, QFormLayout, QHBoxLayout, QCheckBox
from PyQt5.QtWidgets import QSpinBox, QDoubleSpinBox, QPushButton, QVBoxLayout
from PyQt5.QtCore import Qt
import numpy


class PlotScalingDialog(QDialog):
    """
    Dialog that allows the user to set the scale for the X axis
    """
    def __init__(self, parent, currentScale, data):
        QDialog.__init__(self, parent)

        minScale, maxScale = currentScale

        self.setWindowTitle("Plot Scaling")

        self.formLayout = QFormLayout()

        self.minAutoCheck = QCheckBox()
        self.minAutoCheck.setText("Auto")

        floatingPoint = numpy.issubdtype(data.dtype, numpy.floating)
        if floatingPoint:
            info = numpy.finfo(data.dtype)
        else:
            info = numpy.iinfo(data.dtype)

        if floatingPoint:
            self.minValueSpin = QDoubleSpinBox()
            self.minValueSpin.setRange(info.min, info.max)
        else:
            self.minValueSpin = QSpinBox()
            self.minValueSpin.setRange(info.min, info.max)
        self.minAutoCheck.stateChanged.connect(self.onMinAuto)

        if minScale is None:
            self.minAutoCheck.setCheckState(Qt.Checked)
            self.minValueSpin.setEnabled(False)
            self.minValueSpin.setValue(data.min())
        else:
            self.minValueSpin.setValue(minScale)

        self.minLayout = QVBoxLayout()
        self.minLayout.addWidget(self.minAutoCheck)
        self.minLayout.addWidget(self.minValueSpin)
        self.formLayout.addRow("Minimum Value:", self.minLayout)

        self.maxAutoCheck = QCheckBox()
        self.maxAutoCheck.setText("Auto")

        if floatingPoint:
            self.maxValueSpin = QDoubleSpinBox()
            self.maxValueSpin.setRange(info.min, info.max)
        else:
            self.maxValueSpin = QSpinBox()
            self.maxValueSpin.setRange(info.min, info.max)
        self.maxAutoCheck.stateChanged.connect(self.onMaxAuto)

        if maxScale is None:
            self.maxAutoCheck.setCheckState(Qt.Checked)
            self.maxValueSpin.setEnabled(False)
            self.maxValueSpin.setValue(data.max())
        else:
            self.maxValueSpin.setValue(maxScale)

        self.maxLayout = QVBoxLayout()
        self.maxLayout.addWidget(self.maxAutoCheck)
        self.maxLayout.addWidget(self.maxValueSpin)
        self.formLayout.addRow("Maximum Value:", self.maxLayout)

        self.okButton = QPushButton()
        self.okButton.setText("OK")
        self.okButton.clicked.connect(self.accept)

        self.cancelButton = QPushButton()
        self.cancelButton.setText("Cancel")
        self.cancelButton.clicked.connect(self.reject)

        self.buttonLayout = QHBoxLayout()
        self.buttonLayout.addWidget(self.okButton)
        self.buttonLayout.addWidget(self.cancelButton)

        self.mainLayout = QVBoxLayout(self)
        self.mainLayout.addLayout(self.formLayout)
        self.mainLayout.addLayout(self.buttonLayout)
        self.setLayout(self.mainLayout)

    def onMaxAuto(self, state):
        "Called when auto state changed"
        self.maxValueSpin.setEnabled(state != Qt.Checked)
        
    def onMinAuto(self, state):
        "Called when auto state changed"
        self.minValueSpin.setEnabled(state != Qt.Checked)

    def getScale(self):
        """
        Return the tupe of min, max scaling. None for Auto
        """
        if self.minAutoCheck.isChecked():
            minValue = None
        else:
            minValue = self.minValueSpin.value()

        if self.maxAutoCheck.isChecked():
            maxValue = None
        else:
            maxValue = self.maxValueSpin.value()
        
        return (minValue, maxValue)



"""
This module contains the CoordManager class, which manages 
the relationship between coordinates for display and coordinates 
in the raster
"""
# This file is part of 'TuiView' - a simple Raster viewer
# Copyright (C) 2012  Sam Gillingham
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


class CoordManager(object):
    """
    base class for the layer's coordmgr instance. Derived separately
    for vector and raster layers.
    """
    def __init__(self):
        # The size of the display window, in display coords
        self.dspWidth = None
        self.dspHeight = None

    def setDisplaySize(self, width, height):
        """
        Set display size in units of display coordinate system
        """
        self.dspWidth = width
        self.dspHeight = height

    def getWorldExtent(self):
        """
        Gets the extent in world coords as a 4 element tuple.
        To be implmented by derived class
        """
        raise NotImplementedError("getWorldExtent needs to be overridden")

    def setWorldExtent(self, extent):
        """
        Sets the extent in world coords as a 4 element tuple.
        To be implmented by derived class
        """
        raise NotImplementedError("setWorldExtent needs to be overridden")

    def getFullWorldExtent(self):
        """
        Gets the full extent of the dataset in world coords as a 4 element tuple.
        To be implmented by derived class
        """
        raise NotImplementedError("getFullWorldExtent needs to be overridden")

    def getWorldCenter(self):
        """
        Gets the center of the extent in world coords
        """
        (left, top, right, bottom) = self.getWorldExtent()
        x = left + (right - left) / 2.0
        y = bottom + (top - bottom) / 2.0
        return x, y

    def setWorldCenter(self, wldX, wldY):
        """
        Sets the center of the extent in world coords
        """
        currentX, currentY = self.getWorldCenter()
        diffX = wldX - currentX
        diffY = wldY - currentY
        (left, top, right, bottom) = self.getWorldExtent()
        extents = (left + diffX, top + diffY, right + diffX, bottom + diffY)
        self.setWorldExtent(extents)


class VectorCoordManager(CoordManager):
    """
    Manages coords for a vector layer
    """
    def __init__(self):
        CoordManager.__init__(self)
        self.extent = None
        self.fullExtent = None
        self.metersperpix = None

    def recalc(self):
        """
        Recalculate self.metersperpix
        Called when extents or display size changes
        """
        if self.extent is not None and self.dspWidth is not None:
            metersaccross = self.extent[2] - self.extent[0]
            self.metersperpix = metersaccross / self.dspWidth

    def setDisplaySize(self, width, height):
        """
        derived implementation - calls recalculates extent
        """
        CoordManager.setDisplaySize(self, width, height)
        if self.extent is not None:
            (left, top, right, bottom) = self.extent
            self.extent = (left, top, left + self.metersperpix * width, 
                top - self.metersperpix * height)

    def getWorldExtent(self):
        "Get extent in world coords"
        return self.extent

    def setWorldExtent(self, extent):
        "Set extent in world coords"
        self.extent = extent
        self.recalc()

    def getFullWorldExtent(self):
        "full extent of dataset"
        return self.fullExtent

    def setFullWorldExtent(self, extent):
        "sets the full extent of dataset"
        self.fullExtent = extent

    def world2display(self, wldX, wldY):
        """
        convert world coords to display coords
        returns None if outside
        May have C implementation also. Not sure yet.
        """
        display = None
        if self.extent is not None:
            xoff = wldX - self.extent[0]
            yoff = self.extent[1] - wldY
            if xoff >= 0 and yoff >= 0:
                dspX = xoff / self.metersperpix
                dspY = yoff / self.metersperpix
                if dspX < self.dspWidth and dspY < self.dspHeight:
                    display = (dspX, dspY)
        return display

    def display2world(self, dspX, dspY):
        """
        convert display coords to world coords
        """
        world = None
        if self.extent is not None:
            xoff = dspX * self.metersperpix
            yoff = dspY * self.metersperpix
            world = (self.extent[0] + xoff, self.extent[1] - yoff)

        return world

    def recalcBottomRight(self):
        "I don't think we need to do anything here"
        pass


class RasterCoordManager(CoordManager):
    """
    Manage the relationship between the coordinate system used
    for display and the other coordinate systems used in the raster 
    file. An instance of this class represents the current relationship
    for a single raster, for a single display. 
    
    Methods are provided for updating the transformation(s), and for 
    transforming between the different coordinate systems. 
    
    Coordinate systems involved are:
        display coords - this notionally corresponds to the screen pixels,
                         although technically it is the units which Qt exposes
                         as its viewport coordinates
        pixel coords   - this is the pixel row/column coordinates in the
                         raster file, using the GDAL conventions
        world coords   - this is the projected coordinate system of the raster 
                         file, using the GDAL coordinates. 
    
    In all cases, coordinate pairs are given with the horizontal coordinate first, 
    i.e. (x, y), even when referring to row/col pairs. Thus, a row/col pair
    will be given as (col, row). 
                         
    """
    def __init__(self):
        CoordManager.__init__(self)
        # The raster row/col which is to live in the top-left 
        # corner of the display
        self.pixTop = None
        self.pixLeft = None
        # And the bottom-right
        self.pixBottom = None
        self.pixRight = None
        # Ratio of raster pixels to display pixels. 
        # This defines the zoom level. 
        self.imgPixPerWinPix = None
        # GDAL geotransform array, which defines relationship between
        # pixel and world coords
        self.geotransform = None
        # size of the raster
        self.datasetSizeX = None
        self.datasetSizeY = None
    
    def __str__(self):
        """
        For debugging, so I can see what I am set to
        """
        return ("dw:%s dh:%s pt:%s pl:%s pb:%s pr:%s z:%s gt:%s" % (
            self.dspWidth, self.dspHeight, 
            self.pixTop, self.pixLeft, self.pixBottom, self.pixRight, 
            self.imgPixPerWinPix, self.geotransform))
    
    def setTopLeftPixel(self, leftcol, toprow):
        """
        Set row/col of the top/left pixel to display. Args are pixel 
        row/column numbers
        """
        self.pixTop = toprow
        self.pixLeft = leftcol
    
    def setGeoTransformAndSize(self, transform, xsize, ysize):
        """
        Set the GDAL geotransform array and size
        """
        self.geotransform = transform
        self.datasetSizeX = xsize
        self.datasetSizeY = ysize
    
    def calcZoomFactor(self, right, bottom):
        """
        Calculate the zoom factor, given the currently set top/left
        pixel to display, and the bottom/right pixel desired in display,
        for the currently set display size. 
        
        The zoom factor is calculated to come as close as possible
        to display the given section of  the raster, with the current
        display size, but will correct for any difference in aspect ratio
        between the display window and the desired region of the raster. 
        This means that the given bottom/right values are not always
        actually displayed. One of them, either bottom or right, will 
        be maintained, but the other will be adjusted to match the 
        aspect ratio of the display. For this reason, the bottom/right 
        values are not stored on the object, but instead the calculated 
        zoom factor is stored. The whole of the desired region will be 
        fitted into the display window. 
        
        """
        displayAspectRatio = self.dspWidth / self.dspHeight
        rastWidth = right - self.pixLeft
        rastHeight = bottom - self.pixTop
        # workaround for user drawing 
        # horizontal or vertical lines which 
        # causes problems below
        if rastWidth == 0:
            rastWidth = 1
        if rastHeight == 0:
            rastHeight = 1
        rastAspectRatio = rastWidth / rastHeight
        
        if rastAspectRatio < displayAspectRatio:
            rastWidth = displayAspectRatio * rastHeight
            right = self.pixLeft + rastWidth
        elif rastAspectRatio > displayAspectRatio:
            rastHeight = rastWidth / displayAspectRatio
            bottom = self.pixTop + rastHeight
        
        self.imgPixPerWinPix = (right - self.pixLeft) / self.dspWidth
        self.pixBottom = bottom
        self.pixRight = right
    
    def recalcBottomRight(self):
        """
        Called when the window shape has changed. The pixBottom and pixRight
        values are recalculated, based on the new window shape and the existing 
        zoom factor
        
        """
        self.pixRight = self.pixLeft + self.imgPixPerWinPix * self.dspWidth 
        self.pixBottom = self.pixTop + self.imgPixPerWinPix * self.dspHeight 
    
    def setZoomFactor(self, imgPixPerWinPix):
        """
        Set the zoom to the given value of imgPixPerWinPix. 
        Will then recalcBottomRight(). 
        
        """
        self.imgPixPerWinPix = imgPixPerWinPix
        self.recalcBottomRight()
    
    def display2pixel(self, x, y):
        """
        Convert from display units to raster row/col. Returns
        a tuple (col, row), as floats. 
        """
        col = self.pixLeft + x * self.imgPixPerWinPix
        row = self.pixTop + y * self.imgPixPerWinPix
        return (col, row)
    
    def pixel2display(self, col, row):
        """
        Convert raster row/col to display units. Returns
        a tuple of (x, y). These are int values, as that appears to be
        all Qt will ever deal with. 
        """
        x = int((col - self.pixLeft) / self.imgPixPerWinPix)
        y = int((row - self.pixTop) / self.imgPixPerWinPix)
        return (x, y)

    def pixel2displayF(self, col, row):
        """
        Convert raster row/col to display units. Returns
        a tuple of (x, y). This version returns floats
        """
        x = (col - self.pixLeft) / self.imgPixPerWinPix
        y = (row - self.pixTop) / self.imgPixPerWinPix
        return (x, y)

    def pixel2world(self, col, row):
        """
        Convert raster row/col to world coordinate system. Returns a
        tuple of floats (x, y)
        """
        gt = self.geotransform
        x = gt[0] + col * gt[1] + row * gt[2]
        y = gt[3] + col * gt[4] + row * gt[5]
        return (x, y)
    
    def world2pixel(self, x, y):
        """
        Convert world x,y coordinates to raster row/col. Returns
        a tuple (col, row), as floats. 
        """
        gt = self.geotransform

        # Classic 2x2 matrix inversion
        det = gt[1] * gt[5] - gt[2] * gt[4]
        col = (gt[5] * (x - gt[0]) - gt[2] * (y - gt[3])) / det
        row = (-gt[4] * (x - gt[0]) + gt[1] * (y - gt[3])) / det
        return (col, row)
        
    def display2world(self, dspX, dspY):
        """
        Convert display (x, y) to world coordinates. Returns
        a tuple of floats (x, y), in the world coordinate system
        """
        (col, row) = self.display2pixel(dspX, dspY)
        (wldX, wldY) = self.pixel2world(col, row)
        return (wldX, wldY)
    
    def world2display(self, wldX, wldY):
        """
        Convert world (x, y) to display coordinates. Returns
        a tuple of int values (x, y) in display coordinate system
        """
        (col, row) = self.world2pixel(wldX, wldY)
        (dspX, dspY) = self.pixel2display(col, row)
        return (dspX, dspY)

    def getWorldExtent(self):
        """
        Get the extent of the displayed area in world coords
        A 4 element tuple is returned.
        """
        (left, top) = self.display2world(0, 0)
        (right, bottom) = self.display2world(self.dspWidth, self.dspHeight)
        return (left, top, right, bottom)

    def setWorldExtent(self, extent):
        """
        Sets the world extent as a 4 element tuple
        """
        (leftWorld, topWorld, rightWorld, bottomWorld) = extent
        (left, top) = self.world2pixel(leftWorld, topWorld)
        self.setTopLeftPixel(left, top)
        (right, bottom) = self.world2pixel(rightWorld, bottomWorld)
        self.calcZoomFactor(right, bottom)

    def getFullWorldExtent(self):
        """
        Gets the full extent of the dataset in world coords as a 
        4 element tuple.
        """
        (left, top) = self.pixel2world(0, 0)
        (right, bottom) = self.pixel2world(self.datasetSizeX - 1, 
            self.datasetSizeY - 1)
        return (left, top, right, bottom)



"""
Check if file within a zip archive and return apropriate GDAL path if so.

See GDAL documentation on reading files within a zip archive.

http://trac.osgeo.org/gdal/wiki/UserDocs/ReadInZip
"""
# This file is part of 'TuiView' - a simple Raster viewer
# Copyright (C) 2012  Sam Gillingham
#
# archivereader.py written by Terry Cain (2014), Plymouth Marine Laboratory 
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import zipfile

ALLOWED_FILE_EXTENSIONS = ('.bil', '.bsq', '.bin', '.env', '.dem', '.tif', '.asc', '.img')


def file_list_to_archive_strings(filenames):
    """
    Modifies any archive filenames in the list to allow them to be opened using a GDAL virtual filesystem.

    :param filenames: A list of filenames
    :type filenames: list of str

    :return: The list of filenames passed in, with any supported archive prepended with the appropriate string.
    :rtype: list of str
    """
    output_filename_list = []

    for filename in filenames:
        if filename.endswith('.gz') and filename.find(".tar.gz") == -1:  # can process gz files but opening tar files in python (tarfile) takes ages
            output_filename_list.append(gz_to_file(filename))
        elif filename.endswith('.zip'):
            output_filename_list.extend(zip_to_file(filename))
        else:
            output_filename_list.append(filename)

    return output_filename_list


def gz_to_file(filepath):
    """
    Converts a path ending in .gz so that it can be opened with GDAL

    :param filepath: Filepath ending in .gz
    :type filepath: str

    :return: Filepath prepended with gzip virtual filesystem string
    :rtype: str
    """
    return "/vsigzip/" + filepath


def zip_to_file(filepath):
    """
    Takes in a zip filepath and if the zip contains files that can be opened with GDAL then the filepath will be converted
    so that it can be opened without extraction.

    :param filepath: Filepath ending in .zip
    :type filepath: str

    :return: A list of files prepended with the zip virtual filesystem string
    :rtype: list of str
    """
    zip_file_list = []
    if zipfile.is_zipfile(filepath):
        try:
            zip_file = zipfile.ZipFile(filepath)
            zip_file_contents = ['/vsizip/{0}/{1}'.format(filepath, zip_info_object.filename) for zip_info_object in zip_file.filelist if zip_info_object.filename.endswith(ALLOWED_FILE_EXTENSIONS)]
            zip_file_list.extend(zip_file_contents)
            zip_file.close()
        except zipfile.BadZipfile:
            pass

    return zip_file_list

